
#include <Animationrig.h>

namespace Animrig {
	// key
	AnimKey* AnimKey::createKeyFromLayerType(LayerType layertype, int frame_)
	{
		AnimKey *key = nullptr;
		switch (layertype)
		{
		case LayerType::AnimBlend4: key = new AnimBlend4Key(); break;
		case LayerType::AnimBlend1: key = new AnimBlend1Key(); break;
		case LayerType::SeekBlend4: key = new SeekBlend4Key(); break;
		case LayerType::SeekBlend1: key = new SeekBlend1Key(); break;
		case LayerType::Transform: key = new TransformKey(); break;
		case LayerType::Event: key = new EventKey(); break;
		case LayerType::Priority: key = new PriorityKey(); break;
		}
		if (key == nullptr)
		{
			throw exception("CreateKeyFromLayerType : unsupport type.");
		}
		key->frame = frame_;
		return key;
	}
	// layer
	AnimLayer* AnimLayer::createFromLayerType(LayerType layertype) {
		AnimLayer *layer = nullptr;
		switch (layertype)
		{
		case LayerType::AnimBlend4: layer = new AnimBlend4Layer(); break;
		case LayerType::AnimBlend1: layer = new AnimBlend1Layer(); break;
		case LayerType::SeekBlend4: layer = new SeekBlend4Layer(); break;
		case LayerType::SeekBlend1: layer = new SeekBlend1Layer(); break;
		case LayerType::Transform: layer = new TransformLayer(); break;
		case LayerType::Event: layer = new EventLayer(); break;
		case LayerType::Priority: layer = new PriorityLayer(); break;
		}
		if (layer == nullptr)
		{
			throw exception("CreateFromLayerType : unsupport layer type.");
		}
		return layer;
	}
	// object
	AnimObject* AnimObject::createFromObjectType(ObjectType objecttype)
	{
		AnimObject *obj = nullptr;
		switch (objecttype)
		{
		case ObjectType::Null: obj = new NullObject(); break;
		case ObjectType::Spine: obj = new SpineObject(); break;
		}
		if (obj == nullptr)
		{
			throw exception("AddObject : unsupport object.");
		}
		return obj;
	}
}